//======================================================================
//
// Author        : ADI - CAST 
//
// Date          : Sep 2006
//
// File          : AD7450.c
//
// Description   : use the spi function of MicroConverter(ADuC812) 
//					to control the ADC AD7450
//
//					SPI setting:  Fosc/4, CPHA=0, CPOL=1, Master Mode
//
//					(1)use tmr2 in timer function to trigger ADC in 1 SPS;
//					(2)use int0 to trigger the Power-Down/Power-Up Mode
//
//					Reference : ADuC843's example c-code "adcpin.c"           
//
//======================================================================

#include <stdio.h>
#include <ADuC812.h>

sbit SS = P3^5;
sbit LED = P3^4;

unsigned char ADH,ADL;	//adc result

void UART_Init()
{
	TMOD = 0x20;
	TH1 = 0xFD;
	TL1 = 0xFD;
	SCON = 0x52;
	TR1 = 1;
}

void UART_sendchar(unsigned char c)
{
	while(!TI);
	TI = 0;
	SBUF = c;
}

void SPI_Init()
{
	SPICON = 0x38;	//configure SPI port for:
                    //Fosc/4, CPHA=0, CPOL=1, master
}

unsigned char SPI_read_byte()
{
	unsigned char dat=0x11;
	SPIDAT = dat;	//start spi receive
	while(!ISPI);	//wait spi complete
	ISPI = 0;
	return SPIDAT;
}

void ADC_NormalRead()	//ADC normal mode
{
	SS = 0;
	ADH = SPI_read_byte();
	ADL = SPI_read_byte();
	SS = 1;
}

void ADC_PowerDown()	//ADC power-down mode
{
	SS = 0;
	ADH = SPI_read_byte();
	SS = 1;
}

void ADC_PowerUp()		//ADC power-up mode
{
	ADC_NormalRead();				//dummy read
}

void tmr2_interrupt () interrupt 5	//tmr2
{
	static unsigned int cc = 1000;
	TF2 = 0;

	cc--;
	if(!cc)				//ADC in 1 SPS
	{
		cc = 1000;
		LED ^= 1;
		ADC_NormalRead();
		UART_sendchar(ADH);	//trasmit result
		UART_sendchar(ADL);
	}
    
    

}

void int0_interrupt () interrupt 0	//int0
{
	unsigned char i,j;
	static flag = 0;
	ET2 = 0;
	for(i=0;i<200;i++)
		for(j=0;j<200;j++)
			;
	if(!flag)		//Power-Down Mode
	{
		flag = 1;
		LED = 1;
		ADC_PowerDown();
		UART_sendchar(0xDD);
	}
	else			//Power-Up Mode
	{
		flag = 0;
		LED = 0;
		ADC_PowerUp();
		UART_sendchar(0xAA);
		ET2 = 1;
	}
}



void main()
{
	SPI_Init();
	UART_Init();

	ET2 = 1;
	
	RCAP2H = 0xFC;	
	RCAP2L = 0x66;
	TH2 = 0xFC;
	TL2 = 0x66;		

	EX0 = 1;
	IT0 = 1;		//enable int0

	EA = 1;
	T2CON = 0x04;	//tmr2 in timer function, 1kHz, auto reload

	while(1);
}

